@@ -1,12 +1,19 @@
 #include "ncrypt.h"
 #include "decrypt_file.h"
 
-void decrypt_file(FILE* ifp, int fsize, FILE* ofp, int algo)
+void decrypt_file(FILE* ifp, FILE* ofp, int algo)
 {
     BYTE block[4 * 4];
+    long int fsize = 0;
     int i, writesize = 0, size = 0;
 
-    while (fsize > 0)
+    size = fread(&fsize, sizeof(long int), 1, ifp);
+    if (size <= 0)
+    {
+        fprintf(stderr,"*** Error reading data\n");
+        exit(-1);
+    }
+    while (1)
     {
         guaranteed_memset(block, 0, 16);
         size = fread(block, 1, 16, ifp);
@@ -24,18 +31,23 @@ void decrypt_file(FILE* ifp, int fsize, FILE* ofp, int algo)
             break;
         }
         /* on last block, eliminate the padding if it exists */
-        if(fsize == 16)
+        if(fsize < 17) /* last block */
         {
-            for(i=0; i<16; i++)
-            if(block[i] != 0) writesize = i+1;
+            if (fwrite(block, (int)fsize, 1, ofp) != 1)
+            {
+                fprintf(stderr,"** ERROR writing data from last block\n");
+                exit(-1);
+            }
+            break;
         }
-        else writesize = 16;
-        if (fwrite(block, writesize, 1, ofp) != 1)
+        else /* otherwise, normal writing to decrypt file */
         {
-            fprintf(stderr,"** ERROR writing data\n");
-            exit(-1);
+            if (fwrite(block, 16, 1, ofp) != 1)
+            {
+                fprintf(stderr,"** ERROR writing data\n");
+                exit(-1);
+            }
         }
-        if (writesize != 16) break;
         fsize -= size;
     }
 }
