@@ -8,7 +8,7 @@
  * Rancier <mephis5@softhome.net>. The Twofish code was taken from Doug 
  * Whiting's C implementation from www.counterpane.com.
  * 
- * $Id: ncrypt.c,v 1.8 2004/03/17 05:02:56 s-nomad Exp $
+ * $Id: ncrypt.c,v 1.9 2004/08/09 14:51:03 s-nomad Exp $
  */
 
 #include <stdio.h>
@@ -154,10 +154,12 @@ int wipe(int wipe_mode, char * ifile)
 
 int encrypt_it(FILE* ifp, FILE* ofp)
 {
+    struct stat statbuf;
     char* pass = get_encryption_password();
     initialize_crypto(pass,1);
     free(pass);
-    encrypt_file(ifp,ofp,NCRYPT.algo);
+    fstat(fileno(ifp),&statbuf);
+    encrypt_file(ifp,statbuf.st_size,ofp,NCRYPT.algo);
     return 0;
 }
 
@@ -165,14 +167,10 @@ int encrypt_it(FILE* ifp, FILE* ofp)
 
 int decrypt_it(FILE* ifp, FILE* ofp)
 {
-    struct stat statbuf;
-    int fsize = 0;
     char* pass = get_decryption_password();
     initialize_crypto(pass,0);
     free(pass);
-    fstat(fileno(ifp),&statbuf);
-    fsize = (int)statbuf.st_size;
-    decrypt_file(ifp,fsize,ofp,NCRYPT.algo);
+    decrypt_file(ifp,ofp,NCRYPT.algo);
     return 0;
 }
 
